#!/usr/bin/perl -w
# Tossed this together in about 15 minutes so don't expect it to be pretty!
# Recommend you run it under screen or tmux
# Requires the following packages: libwww-perl libdbd-sqlite3-perl sqlite3
use LWP::Simple qw($ua get);
$ua->agent("TMWOnlineLog/1.1");
use DBI;
use strict;
my %names;
my %old_names;
my %ontime;

# Configurable: How often to check online list, in seconds
my $interval = 5;

die("Interval out of range") if ($interval < 5 || $interval > 1200);

my $newdb++ if (!-f "players.db");
my $dbh = DBI->connect("dbi:SQLite:dbname=players.db") or die;

if (defined($newdb)) {
  print("New database? Creating tables and indexes\n");
  $dbh->do("create table log ('name' varchar(26), action integer, rank integer, time integer)");
  $dbh->do("create table ontime ('name' varchar(26), ontime integer, primary key (name))");
  $dbh->do("create index nameidx on log (name)");
  $dbh->do("create index timeidx on log (time)");
  $dbh->do("create index rankidx on log (rank)");
  $dbh->do("create index ontimeidx on ontime (ontime)");
}
$SIG{'INT'} = \&clean_shutdown;

my $log = $dbh->prepare('INSERT INTO log VALUES(?, ?, ?, ?)');

while(1) {
  my $now = time();
  %old_names = %names;
  undef(%names);
  get_online_list();

  foreach(keys %names) {
    # Add to the ontime of everyone who's currently logged in
    if (defined($ontime{$_})) { $ontime{$_} = $ontime{$_} + $interval; }
    else { $ontime{$_} = 0; }

    # Figure out who has logged in in the past interval
    if (!defined($old_names{$_})) {
      $log->execute($_, 1, $names{$_}, $now);
      print "Logged in: $_ $names{$_}\n";
    }
  }

  foreach (keys %old_names) {
    # Figure out who logged out
    if(!defined($names{$_})) {
      $log->execute($_, 0, $old_names{$_}, $now);
      print "Logged out: $_ ($old_names{$_})\n";
    }
  }

  # Save the ontimes every once in a while..
  save_ontimes() if (($now % 1200) < $interval+2);

  sleep($interval);
}
	
sub get_online_list {
  my ($ok, $content, @lines, $pos);
  $content = get("http://server.themanaworld.org/online.txt")
    or warn("WARNING: Could not contact server\n"),return;

  $content =~ s/\r//;
  @lines = split(/\n/, $content);
  $pos = 0;
  foreach (@lines) {
    chomp;
    s/\s+$//;
    s/\s+\(GM\)$//;
    $ok++,next if (/----------/);
    next unless $ok;
    last if (/^$/ && $ok);
    $names{$_} = $pos++;
  }
}

# Why am I not using tied hashes? Too lazy to turn back now!
sub load_ontimes {
  my $sth = $dbh->prepare("SELECT * from ontime");
  $sth->execute;
  while (my @data = $sth->fetchrow_array()) {
    $ontime{$data[0]} = $data[1];
  }
}

sub save_ontimes {
  my $sth = $dbh->prepare("replace into ontime values(?, ?)" );
  foreach (keys %ontime) { $sth->execute($_, $ontime{$_}); }
}

sub clean_shutdown {
  my $now = time();
  print("Shutting down cleanly..\n");

  # Log action=3 for everyone who's currently logged in
  foreach (keys %names) {
    $log->execute($_, 3, $names{$_}, $now);
  }

  save_ontimes();

  exit(0);
}

